/*
  Copyright (c) 2014, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include "MapContext.hpp"


namespace RPGEdit {

namespace Control {

MapContext::MapContext(EventBus *eventBus0, InputHandler *inputHandler0, View::MapView *view0, const std::shared_ptr<const Model::Map> &map0)
	: eventBus(eventBus0), inputHandler(inputHandler0), view(view0), map(map0) {
	for (const Model::Entity &entity : map->getEntities())
		entities.emplace_back(new Model::Entity(entity));

	playerEntity = new Model::Entity("square");
	playerEntity->moveTo(Model::Position{8, 8});

	entities.emplace_back(playerEntity);

	view->updateEntities(entities);


	inputHandler->registerListener(
		[this] (uint16_t key, bool pressed, uint64_t time) {
			if (pressed)
				keyPressed(key, time);
		}
	);
}

void MapContext::movePlayer(Model::Direction dir, uint64_t time) {
	if (playerEntity->hasTransition())
		return;

	playerEntity->move(dir, time, time+250);

	eventBus->enqueue(
		[=] {
			playerEntity->finishTransition();
			movePlayerContinue(time+250);
		},
		time+250
	);
}

void MapContext::movePlayerContinue(uint64_t time) {
	if (inputHandler->isKeyPressed(SDL_SCANCODE_UP))
		movePlayer(Model::NORTH, time);
	else if (inputHandler->isKeyPressed(SDL_SCANCODE_RIGHT))
		movePlayer(Model::EAST, time);
	else if (inputHandler->isKeyPressed(SDL_SCANCODE_DOWN))
		movePlayer(Model::SOUTH, time);
	else if (inputHandler->isKeyPressed(SDL_SCANCODE_LEFT))
		movePlayer(Model::WEST, time);
}


void MapContext::keyPressed(uint16_t key, uint64_t time) {
	switch (key) {
	case SDL_SCANCODE_UP:
		movePlayer(Model::NORTH, time);
		break;

	case SDL_SCANCODE_RIGHT:
		movePlayer(Model::EAST, time);
		break;

	case SDL_SCANCODE_DOWN:
		movePlayer(Model::SOUTH, time);
		break;

	case SDL_SCANCODE_LEFT:
		movePlayer(Model::WEST, time);
	}
}


}

}
