/*
  Copyright (c) 2014, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#pragma once

#include <cstdint>
#include <memory>
#include <stdexcept>
#include <vector>

#include "Entity.hpp"


namespace RPGEdit {

namespace Model {

class Map {
private:
	std::vector<std::string> tileset;

	size_t width, height;
	std::vector<std::vector<uint32_t>> tiles;
	std::vector<Entity> entities;


	Map(size_t width0, size_t height0, size_t layers)
		: width(width0), height(height0) {
		for (size_t i = 0; i < layers; i++)
			tiles.emplace_back(width*height);
	}

public:
	std::vector<std::string> & getTileset() {
		return tileset;
	}

	const std::vector<std::string> & getTileset() const {
		return tileset;
	}

	std::vector<Entity> & getEntities() {
		return entities;
	}

	const std::vector<Entity> & getEntities() const {
		return entities;
	}

	size_t getWidth() const {
		return width;
	}

	size_t getHeight() const {
		return height;
	}

	size_t getLayerCount() const {
		return tiles.size();
	}

	void setTileAt(size_t layer, size_t x, size_t y, uint32_t value) {
		if (layer >= tiles.size() || x >= width || y >= height)
			throw std::range_error("Map::setTileAt: bad coordinates");

		tiles[layer][y*width + x] = value;
	}

	uint32_t getTileAt(size_t layer, ssize_t x, ssize_t y) const {
		if (layer >= tiles.size() || x < 0 || (size_t)x >= width || y < 0 || (size_t)y >= height)
			return 0;

		return tiles[layer][y*width + x];
	}

	static std::shared_ptr<Map> load(const std::string &name);
};

}

}
